"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const chai_1 = require("chai");
const command_manager_1 = require("./command_manager");
describe('ContextMenuCommandManager', () => {
    class ManagerImplementation extends command_manager_1.ContextMenuCommandManager {
        get_rank(command) {
            return super.get_rank(command);
        }
        get current_adapter() {
            return undefined;
        }
    }
    let manager;
    let base_command = {
        id: 'cmd',
        execute: () => {
            // nothing here het
        },
        is_enabled: () => {
            return true;
        },
        label: 'Command'
    };
    describe('#get_rank()', () => {
        it('uses in-group (relative) positioning by default', () => {
            manager = new ManagerImplementation(null, null, null, 0, 5);
            let rank = manager.get_rank(base_command);
            chai_1.expect(rank).to.equal(0);
            rank = manager.get_rank(Object.assign({}, base_command, { rank: 1 }));
            chai_1.expect(rank).to.equal(1 / 5);
            manager = new ManagerImplementation(null, null, null, 1, 5);
            rank = manager.get_rank(Object.assign({}, base_command, { rank: 1 }));
            chai_1.expect(rank).to.equal(1 + 1 / 5);
        });
    });
    it('respects is_rank_relative value', () => {
        manager = new ManagerImplementation(null, null, null, 0, 5);
        let rank = manager.get_rank(Object.assign({}, base_command, { rank: 1, is_rank_relative: false }));
        chai_1.expect(rank).to.equal(1);
        rank = manager.get_rank(Object.assign({}, base_command, { rank: 1, is_rank_relative: true }));
        chai_1.expect(rank).to.equal(1 / 5);
    });
});
