"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeMirror = require("codemirror");
const utils_1 = require("../../utils");
class CodeMirrorAdapter {
    constructor(editor, virtual_document, jupyterlab_components, features = new Array()) {
        this.editor = editor;
        this.virtual_document = virtual_document;
        this.jupyterlab_components = jupyterlab_components;
        this.doc_change_handler = this.saveChange.bind(this);
        this.editor.on('change', this.doc_change_handler);
        this.features = [];
        for (let feature of features) {
            feature.register();
            if (!feature.is_registered) {
                console.warn('The feature ', feature, 'was not registered properly');
            }
            this.features.push(feature);
        }
    }
    updateAfterChange() {
        return __awaiter(this, void 0, void 0, function* () {
            this.jupyterlab_components.remove_tooltip();
            yield utils_1.until_ready(() => this.last_change != null, 30, 22).catch(() => {
                this.invalidateLastChange();
                throw Error('No change obtained from CodeMirror editor within the expected time of 0.66s');
            });
            let change = this.last_change;
            try {
                const root_position = this.editor
                    .getDoc()
                    .getCursor('end');
                let document = this.editor.document_at_root_position(root_position);
                if (this.virtual_document !== document) {
                    return true;
                }
                if (!change || !change.text.length || !change.text[0].length) {
                    // deletion - ignore
                    return true;
                }
                for (let feature of this.features) {
                    feature.afterChange(change, root_position);
                }
                return true;
            }
            catch (e) {
                console.log('updateAfterChange failure');
                console.error(e);
            }
            this.invalidateLastChange();
        });
    }
    invalidateLastChange() {
        this.last_change = null;
    }
    saveChange(doc, change) {
        this.last_change = change;
    }
    remove() {
        for (let feature of this.features) {
            feature.remove();
        }
        this.editor.off('change', this.doc_change_handler);
    }
}
exports.CodeMirrorAdapter = CodeMirrorAdapter;
