"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const feature_1 = require("../feature");
const converter_1 = require("../../../converter");
const utils_1 = require("../../../utils");
class JumpToDefinition extends feature_1.CodeMirrorLSPFeature {
    register() {
        this.connection_handlers.set('goTo', this.handle_jump.bind(this));
        super.register();
    }
    get jumper() {
        return this.jupyterlab_components.jumper;
    }
    handle_jump(location_or_locations) {
        return __awaiter(this, void 0, void 0, function* () {
            let connection = this.connection;
            // some language servers appear to return a single object
            const locations = Array.isArray(location_or_locations)
                ? location_or_locations
                : [location_or_locations];
            // TODO: implement selector for multiple locations
            //  (like when there are multiple definitions or usages)
            //  could use the showHints() or completion frontend as a reference
            if (locations.length === 0) {
                console.log('No jump targets found');
                return;
            }
            console.log('Will jump to the first of suggested locations:', locations);
            let location = locations[0];
            let uri = decodeURI(location.uri);
            let current_uri = connection.getDocumentUri();
            let virtual_position = converter_1.PositionConverter.lsp_to_cm(location.range.start);
            if (utils_1.uris_equal(uri, current_uri)) {
                let editor_index = this.virtual_editor.get_editor_index(virtual_position);
                // if in current file, transform from the position within virtual document to the editor position:
                let editor_position = this.virtual_editor.transform_virtual_to_editor(virtual_position);
                let editor_position_ce = converter_1.PositionConverter.cm_to_ce(editor_position);
                console.log(`Jumping to ${editor_index}th editor of ${uri}`);
                console.log('Jump target within editor:', editor_position_ce);
                this.jumper.jump({
                    token: {
                        offset: this.jumper.getOffset(editor_position_ce, editor_index),
                        value: ''
                    },
                    index: editor_index
                });
            }
            else {
                // otherwise there is no virtual document and we expect the returned position to be source position:
                let source_position_ce = converter_1.PositionConverter.cm_to_ce(virtual_position);
                console.log(`Jumping to external file: ${uri}`);
                console.log('Jump target (source location):', source_position_ce);
                // can it be resolved vs our guessed server root?
                const contents_path = utils_1.uri_to_contents_path(uri);
                if (contents_path) {
                    uri = contents_path;
                }
                else if (uri.startsWith('file://')) {
                    uri = uri.slice(7);
                }
                let jump_data = {
                    editor_index: 0,
                    line: source_position_ce.line,
                    column: source_position_ce.column
                };
                // assume that we got a relative path to a file within the project
                // TODO use is_relative() or something? It would need to be not only compatible
                //  with different OSes but also with JupyterHub and other platforms.
                try {
                    yield this.jumper.document_manager.services.contents.get(uri, {
                        content: false
                    });
                    this.jumper.global_jump(Object.assign({ uri }, jump_data), false);
                    return;
                }
                catch (err) {
                    console.warn(err);
                }
                this.jumper.global_jump(Object.assign({ uri: '.lsp_symlink/' + uri }, jump_data), true);
            }
        });
    }
}
JumpToDefinition.commands = [
    {
        id: 'jump-to-definition',
        execute: ({ connection, virtual_position }) => connection.getDefinition(virtual_position),
        is_enabled: ({ connection }) => connection.isDefinitionSupported(),
        label: 'Jump to definition'
    }
];
exports.JumpToDefinition = JumpToDefinition;
