"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const jl_adapter_1 = require("./jl_adapter");
const completion_1 = require("./components/completion");
const file_editor_1 = require("../../virtual/editors/file_editor");
class FileEditorAdapter extends jl_adapter_1.JupyterLabWidgetAdapter {
    constructor(editor_widget, jumper, app, completion_manager, rendermime_registry, server_root) {
        super(app, editor_widget, rendermime_registry, 'completer:invoke-file', server_root);
        this.completion_manager = completion_manager;
        this.jumper = jumper;
        this.editor = editor_widget.content;
        this.virtual_editor = new file_editor_1.VirtualFileEditor(() => this.language, () => this.language_file_extension, () => this.document_path, this.cm_editor);
        this.connect_contentChanged_signal();
        console.log('LSP: file ready for connection:', this.path);
        this.connect_document(this.virtual_editor.virtual_document).catch(console.warn);
        this.editor.model.mimeTypeChanged.connect(this.reload_connection.bind(this));
    }
    get document_path() {
        return this.widget.context.path;
    }
    get mime_type() {
        return this.editor.model.mimeType;
    }
    get language_file_extension() {
        let parts = this.document_path.split('.');
        return parts[parts.length - 1];
    }
    get ce_editor() {
        return this.editor.editor;
    }
    get cm_editor() {
        return this.ce_editor.editor;
    }
    find_ce_editor(cm_editor) {
        return this.editor.editor;
    }
    connect_completion() {
        this.current_completion_connector = new completion_1.LSPConnector({
            editor: this.editor.editor,
            connections: this.connection_manager.connections,
            virtual_editor: this.virtual_editor
        });
        this.completion_manager.register({
            connector: this.current_completion_connector,
            editor: this.editor.editor,
            parent: this.widget
        });
    }
    get path() {
        return this.widget.context.path;
    }
}
exports.FileEditorAdapter = FileEditorAdapter;
