"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const jl_adapter_1 = require("./jl_adapter");
const notebook_1 = require("../../virtual/editors/notebook");
const utils_1 = require("../../utils");
const completion_1 = require("./components/completion");
const defaults_1 = require("../../magics/defaults");
const defaults_2 = require("../../extractors/defaults");
class NotebookAdapter extends jl_adapter_1.JupyterLabWidgetAdapter {
    constructor(editor_widget, jumper, app, completion_manager, rendermime_registry, server_root) {
        super(app, editor_widget, rendermime_registry, 'completer:invoke-notebook', server_root);
        this.editor = editor_widget.content;
        this.completion_manager = completion_manager;
        this.jumper = jumper;
        this.init_once_ready()
            .then()
            .catch(console.warn);
        this.widget.context.session.kernelChanged.connect((_session, change) => {
            change.newValue.ready
                .then((spec) => __awaiter(this, void 0, void 0, function* () {
                console.log('LSP: Changed to ' +
                    change.newValue.info.language_info.name +
                    ' kernel, reconnecting');
                yield utils_1.until_ready(this.is_ready.bind(this), -1);
                this.reload_connection();
            }))
                .catch(e => {
                console.warn(e);
                // try to reconnect anyway
                this.reload_connection();
            });
        });
    }
    is_ready() {
        return (this.widget.context.isReady &&
            this.widget.content.isVisible &&
            this.widget.content.widgets.length > 0 &&
            this.widget.context.session.kernel !== null);
    }
    get document_path() {
        return this.widget.context.path;
    }
    language_info() {
        try {
            return this.widget.context.session.kernel.info.language_info;
        }
        catch (e) {
            console.warn('Could not get kernel metadata');
            return null;
        }
    }
    get mime_type() {
        let language_metadata = this.language_info();
        if (!language_metadata) {
            // fallback to the code cell mime type if no kernel in use
            return this.widget.content.codeMimetype;
        }
        return language_metadata.mimetype;
    }
    get language_file_extension() {
        let language_metadata = this.language_info();
        if (!language_metadata) {
            return null;
        }
        return language_metadata.file_extension.replace('.', '');
    }
    find_ce_editor(cm_editor) {
        return this.virtual_editor.cm_editor_to_cell.get(cm_editor).editor;
    }
    init_once_ready() {
        return __awaiter(this, void 0, void 0, function* () {
            console.log('LSP: waiting for', this.document_path, 'to fully load');
            yield utils_1.until_ready(this.is_ready.bind(this), -1);
            console.log('LSP:', this.document_path, 'ready for connection');
            this.virtual_editor = new notebook_1.VirtualEditorForNotebook(this.widget, () => this.language, () => this.language_file_extension, defaults_1.language_specific_overrides, defaults_2.foreign_code_extractors, () => this.document_path);
            this.connect_contentChanged_signal();
            this.connect_document(this.virtual_editor.virtual_document).catch(console.warn);
        });
    }
    set_completion_connector(cell) {
        if (this.current_completion_connector) {
            delete this.current_completion_connector;
        }
        this.current_completion_connector = new completion_1.LSPConnector({
            editor: cell.editor,
            connections: this.connection_manager.connections,
            virtual_editor: this.virtual_editor,
            session: this.widget.session
        });
    }
    connect_completion() {
        // see https://github.com/jupyterlab/jupyterlab/blob/c0e9eb94668832d1208ad3b00a9791ef181eca4c/packages/completer-extension/src/index.ts#L198-L213
        const cell = this.widget.content.activeCell;
        this.set_completion_connector(cell);
        const handler = this.completion_manager.register({
            connector: this.current_completion_connector,
            editor: cell.editor,
            parent: this.widget
        });
        this.widget.content.activeCellChanged.connect((notebook, cell) => {
            this.set_completion_connector(cell);
            handler.editor = cell.editor;
            handler.connector = this.current_completion_connector;
        });
    }
}
exports.NotebookAdapter = NotebookAdapter;
