"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const events = __importStar(require("events"));
const protocol = __importStar(require("vscode-languageserver-protocol"));
const vscode_ws_jsonrpc_1 = require("vscode-ws-jsonrpc");
function registerServerCapability(serverCapabilities, registration) {
    const serverCapabilitiesCopy = JSON.parse(JSON.stringify(serverCapabilities));
    const { method, registerOptions } = registration;
    const providerName = method.substring(0, 14) + 'Provider';
    console.log(providerName);
    if (providerName) {
        if (!registerOptions) {
            serverCapabilitiesCopy[providerName] = true;
        }
        else {
            serverCapabilitiesCopy[providerName] = JSON.parse(JSON.stringify(registerOptions));
        }
    }
    else {
        throw new Error('Could not register server capability.');
    }
    return serverCapabilitiesCopy;
}
exports.registerServerCapability = registerServerCapability;
function unregisterServerCapability(serverCapabilities, unregistration) {
    const serverCapabilitiesCopy = JSON.parse(JSON.stringify(serverCapabilities));
    const { method } = unregistration;
    const providerName = method.substring(0, 14) + 'Provider';
    console.log(providerName);
    delete serverCapabilitiesCopy[providerName];
    return serverCapabilitiesCopy;
}
exports.unregisterServerCapability = unregisterServerCapability;
/**
 * Changes as compared to to origin:
 *  - markdown is preferred over plaintext
 *  - informative members are public and others are protected, not private
 *  - onServerInitialized() was extracted; it also emits a message once connected
 */
class LspWsConnectionReimplemented extends events.EventEmitter {
    constructor(options) {
        super();
        this.isConnected = false;
        this.isInitialized = false;
        this.documentVersion = 0;
        this.documentInfo = options;
    }
    /**
     * Initialize a connection over a web socket that speaks the LSP protocol
     */
    connect(socket) {
        this.socket = socket;
        vscode_ws_jsonrpc_1.listen({
            webSocket: this.socket,
            logger: new vscode_ws_jsonrpc_1.ConsoleLogger(),
            onConnection: (connection) => {
                connection.listen();
                this.isConnected = true;
                this.connection = connection;
                this.sendInitialize();
                this.connection.onNotification('textDocument/publishDiagnostics', (params) => {
                    this.emit('diagnostic', params);
                });
                this.connection.onNotification('window/showMessage', (params) => {
                    this.emit('logging', params);
                });
                this.connection.onRequest('client/registerCapability', (params) => {
                    params.registrations.forEach((capabilityRegistration) => {
                        this.serverCapabilities = registerServerCapability(this.serverCapabilities, capabilityRegistration);
                    });
                    this.emit('logging', params);
                });
                this.connection.onRequest('client/unregisterCapability', (params) => {
                    params.unregisterations.forEach((capabilityUnregistration) => {
                        this.serverCapabilities = unregisterServerCapability(this.serverCapabilities, capabilityUnregistration);
                    });
                    this.emit('logging', params);
                });
                this.connection.onRequest('window/showMessageRequest', (params) => {
                    this.emit('logging', params);
                });
                this.connection.onError(e => {
                    this.emit('error', e);
                });
                this.connection.onClose(() => {
                    this.isConnected = false;
                });
            }
        });
        return this;
    }
    close() {
        if (this.connection) {
            this.connection.dispose();
        }
        this.socket.close();
    }
    getDocumentUri() {
        return this.documentInfo.documentUri;
    }
    sendInitialize() {
        if (!this.isConnected) {
            return;
        }
        const message = {
            capabilities: {
                textDocument: {
                    hover: {
                        dynamicRegistration: true,
                        contentFormat: ['markdown', 'plaintext']
                    },
                    synchronization: {
                        dynamicRegistration: true,
                        willSave: false,
                        didSave: false,
                        willSaveWaitUntil: false
                    },
                    completion: {
                        dynamicRegistration: true,
                        completionItem: {
                            snippetSupport: false,
                            commitCharactersSupport: true,
                            documentationFormat: ['markdown', 'plaintext'],
                            deprecatedSupport: false,
                            preselectSupport: false
                        },
                        contextSupport: false
                    },
                    signatureHelp: {
                        dynamicRegistration: true,
                        signatureInformation: {
                            documentationFormat: ['markdown', 'plaintext']
                        }
                    },
                    declaration: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    definition: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    typeDefinition: {
                        dynamicRegistration: true,
                        linkSupport: true
                    },
                    implementation: {
                        dynamicRegistration: true,
                        linkSupport: true
                    }
                },
                workspace: {
                    didChangeConfiguration: {
                        dynamicRegistration: true
                    }
                }
            },
            initializationOptions: null,
            processId: null,
            rootUri: this.documentInfo.rootUri,
            workspaceFolders: null
        };
        this.connection
            .sendRequest('initialize', message)
            .then(this.onServerInitialized, e => {
            console.warn(e);
        });
    }
    onServerInitialized(params) {
        this.isInitialized = true;
        this.serverCapabilities = params.capabilities;
        console.error('AAA');
        const textDocumentMessage = {
            textDocument: {
                uri: this.documentInfo.documentUri,
                languageId: this.documentInfo.languageId,
                text: this.documentInfo.documentText(),
                version: this.documentVersion
            }
        };
        console.error('AAA');
        this.connection.sendNotification('initialized');
        this.connection.sendNotification('workspace/didChangeConfiguration', {
            settings: {}
        });
        this.connection.sendNotification('textDocument/didOpen', textDocumentMessage);
        this.sendChange();
        console.error('AAA');
        this.emit('serverInitialized', this.serverCapabilities);
    }
    sendChange() {
        if (!this.isConnected) {
            return;
        }
        const textDocumentChange = {
            textDocument: {
                uri: this.documentInfo.documentUri,
                version: this.documentVersion
            },
            contentChanges: [
                {
                    text: this.documentInfo.documentText()
                }
            ]
        };
        this.connection.sendNotification('textDocument/didChange', textDocumentChange);
        this.documentVersion++;
    }
    getHoverTooltip(location) {
        if (!this.isInitialized) {
            return;
        }
        this.connection
            .sendRequest('textDocument/hover', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(params => {
            this.emit('hover', params);
        });
    }
    getCompletion(location, token, triggerCharacter, triggerKind) {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities && this.serverCapabilities.completionProvider)) {
            return;
        }
        this.connection
            .sendRequest('textDocument/completion', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            },
            context: {
                triggerKind: triggerKind || protocol.CompletionTriggerKind.Invoked,
                triggerCharacter
            }
        })
            .then(params => {
            if (!params) {
                this.emit('completion', params);
                return;
            }
            this.emit('completion', 'items' in params ? params.items : params);
        });
    }
    getDetailedCompletion(completionItem) {
        if (!this.isConnected) {
            return;
        }
        this.connection
            .sendRequest('completionItem/resolve', completionItem)
            .then(result => {
            this.emit('completionResolved', result);
        });
    }
    getSignatureHelp(location) {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities && this.serverCapabilities.signatureHelpProvider)) {
            return;
        }
        const code = this.documentInfo.documentText();
        const lines = code.split('\n');
        const typedCharacter = lines[location.line][location.ch];
        if (this.serverCapabilities.signatureHelpProvider &&
            !this.serverCapabilities.signatureHelpProvider.triggerCharacters.indexOf(typedCharacter)) {
            // Not a signature character
            return;
        }
        this.connection
            .sendRequest('textDocument/signatureHelp', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(params => {
            this.emit('signature', params);
        });
    }
    /**
     * Request the locations of all matching document symbols
     */
    getDocumentHighlights(location) {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities &&
            this.serverCapabilities.documentHighlightProvider)) {
            return;
        }
        this.connection
            .sendRequest('textDocument/documentHighlight', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(params => {
            this.emit('highlight', params);
        });
    }
    /**
     * Request a link to the definition of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    getDefinition(location) {
        if (!this.isConnected || !this.isDefinitionSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/definition', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(result => {
            this.emit('goTo', result);
        });
    }
    /**
     * Request a link to the type definition of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    getTypeDefinition(location) {
        if (!this.isConnected || !this.isTypeDefinitionSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/typeDefinition', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(result => {
            this.emit('goTo', result);
        });
    }
    /**
     * Request a link to the implementation of the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    getImplementation(location) {
        if (!this.isConnected || !this.isImplementationSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/implementation', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(result => {
            this.emit('goTo', result);
        });
    }
    /**
     * Request a link to all references to the current symbol. The results will not be displayed
     * unless they are within the same file URI
     */
    getReferences(location) {
        if (!this.isConnected || !this.isReferencesSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/references', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            }
        })
            .then(result => {
            this.emit('goTo', result);
        });
    }
    /**
     * The characters that trigger completion automatically.
     */
    getLanguageCompletionCharacters() {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities &&
            this.serverCapabilities.completionProvider &&
            this.serverCapabilities.completionProvider.triggerCharacters)) {
            return [];
        }
        return this.serverCapabilities.completionProvider.triggerCharacters;
    }
    /**
     * The characters that trigger signature help automatically.
     */
    getLanguageSignatureCharacters() {
        if (!this.isConnected) {
            return;
        }
        if (!(this.serverCapabilities &&
            this.serverCapabilities.signatureHelpProvider &&
            this.serverCapabilities.signatureHelpProvider.triggerCharacters)) {
            return [];
        }
        return this.serverCapabilities.signatureHelpProvider.triggerCharacters;
    }
    /**
     * Does the server support go to definition?
     */
    isDefinitionSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.definitionProvider);
    }
    /**
     * Does the server support go to type definition?
     */
    isTypeDefinitionSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.typeDefinitionProvider);
    }
    /**
     * Does the server support go to implementation?
     */
    isImplementationSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.implementationProvider);
    }
    /**
     * Does the server support find all references?
     */
    isReferencesSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.referencesProvider);
    }
}
exports.LspWsConnectionReimplemented = LspWsConnectionReimplemented;
