"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const connection_base_1 = require("./connection-base");
class LSPConnection extends connection_base_1.LspWsConnectionReimplemented {
    constructor(options) {
        super(options);
        this.closing_manually = false;
    }
    sendSelectiveChange(changeEvent) {
        this._sendChange([changeEvent]);
    }
    sendFullTextChange(text) {
        this._sendChange([{ text }]);
    }
    isRenameSupported() {
        return !!(this.serverCapabilities && this.serverCapabilities.renameProvider);
    }
    rename(location, newName) {
        if (!this.isConnected || !this.isRenameSupported()) {
            return;
        }
        this.connection
            .sendRequest('textDocument/rename', {
            textDocument: {
                uri: this.documentInfo.documentUri
            },
            position: {
                line: location.line,
                character: location.ch
            },
            newName: newName
        })
            .then((result) => {
            this.emit('renamed', result);
        });
    }
    connect(socket) {
        super.connect(socket);
        utils_1.until_ready(() => {
            return this.isConnected;
        }, -1)
            .then(() => {
            let connection = this.connection;
            connection.onClose(() => {
                // @ts-ignore
                this.isConnected = false;
                this.emit('close', this.closing_manually);
            });
        })
            .catch(() => {
            console.error('Could not connect onClose signal');
        });
        return this;
    }
    close() {
        try {
            this.closing_manually = true;
            super.close();
        }
        catch (e) {
            this.closing_manually = false;
        }
    }
    _sendChange(changeEvents) {
        if (!this.isConnected) {
            return;
        }
        let documentInfo = this.documentInfo;
        const textDocumentChange = {
            textDocument: {
                uri: documentInfo.documentUri,
                version: this.documentVersion
            },
            contentChanges: changeEvents
        };
        this.connection.sendNotification('textDocument/didChange', textDocumentChange);
        this.documentVersion++;
    }
    getCompletion(location, token, triggerCharacter, triggerKind) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.isConnected) {
                return;
            }
            if (!(this.serverCapabilities && this.serverCapabilities.completionProvider)) {
                return;
            }
            return new Promise(resolve => {
                this.connection
                    .sendRequest('textDocument/completion', {
                    textDocument: {
                        uri: this.documentInfo.documentUri
                    },
                    position: {
                        line: location.line,
                        character: location.ch
                    },
                    context: {
                        triggerKind: triggerKind,
                        triggerCharacter
                    }
                })
                    .then(params => {
                    if (params) {
                        params = 'items' in params ? params.items : params;
                    }
                    resolve(params);
                });
            });
        });
    }
}
exports.LSPConnection = LSPConnection;
