"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const application_1 = require("@jupyterlab/application");
const apputils_1 = require("@jupyterlab/apputils");
const notebook_1 = require("@jupyterlab/notebook");
const codemirror_1 = require("@jupyterlab/codemirror");
const fileeditor_1 = require("@jupyterlab/fileeditor");
const coreutils_1 = require("@jupyterlab/coreutils");
const docmanager_1 = require("@jupyterlab/docmanager");
const fileeditor_2 = require("@krassowski/jupyterlab_go_to_definition/lib/jumpers/fileeditor");
const notebook_2 = require("@krassowski/jupyterlab_go_to_definition/lib/jumpers/notebook");
require("codemirror/addon/hint/show-hint.css");
require("codemirror/addon/hint/show-hint");
require("../style/index.css");
require("lsp-editor-adapter/lib/codemirror-lsp.css");
const completer_1 = require("@jupyterlab/completer");
const rendermime_1 = require("@jupyterlab/rendermime");
const notebook_3 = require("./adapters/jupyterlab/notebook");
const file_editor_1 = require("./adapters/jupyterlab/file_editor");
const jl_adapter_1 = require("./adapters/jupyterlab/jl_adapter");
const command_manager_1 = require("./command_manager");
var IPaths = application_1.JupyterFrontEnd.IPaths;
const lsp_commands = [].concat(...jl_adapter_1.lsp_features.map(feature => feature.commands));
/**
 * The plugin registration information.
 */
const plugin = {
    id: '@krassowski/jupyterlab-lsp:plugin',
    requires: [
        fileeditor_1.IEditorTracker,
        notebook_1.INotebookTracker,
        coreutils_1.ISettingRegistry,
        apputils_1.ICommandPalette,
        docmanager_1.IDocumentManager,
        completer_1.ICompletionManager,
        rendermime_1.IRenderMimeRegistry,
        IPaths
    ],
    activate: (app, fileEditorTracker, notebookTracker, settingRegistry, palette, documentManager, completion_manager, rendermime_registry, paths) => {
        // temporary workaround for getting the absolute path
        let server_root = paths.directories.serverRoot;
        if (server_root.startsWith('~')) {
            // try to guess the home location:
            let user_settings = paths.directories.userSettings;
            if (user_settings.startsWith('/home/')) {
                server_root = server_root.replace('~', user_settings.substring(0, user_settings.indexOf('/', 6)));
                console.log('Guessing the server root using user settings path', server_root);
            }
            else {
                console.warn('Unable to solve the absolute path: some LSP servers may not work correctly');
            }
        }
        fileEditorTracker.widgetUpdated.connect((sender, widget) => {
            console.log(sender);
            console.log(widget);
            // TODO?
            // adapter.remove();
            // connection.close();
        });
        fileEditorTracker.widgetAdded.connect((sender, widget) => {
            let fileEditor = widget.content;
            if (fileEditor.editor instanceof codemirror_1.CodeMirrorEditor) {
                let jumper = new fileeditor_2.FileEditorJumper(widget, documentManager);
                let adapter = new file_editor_1.FileEditorAdapter(widget, jumper, app, completion_manager, rendermime_registry, server_root);
                command_manager_1.file_editor_adapters.set(fileEditor.id, adapter);
            }
        });
        let command_manager = new command_manager_1.FileEditorCommandManager(app, fileEditorTracker, 'file_editor');
        command_manager.add(lsp_commands);
        notebookTracker.widgetAdded.connect((sender, widget) => {
            // NOTE: assuming that the default cells content factory produces CodeMirror editors(!)
            let jumper = new notebook_2.NotebookJumper(widget, documentManager);
            let adapter = new notebook_3.NotebookAdapter(widget, jumper, app, completion_manager, rendermime_registry, server_root);
            command_manager_1.notebook_adapters.set(widget.id, adapter);
        });
        // position context menu entries after 10th but before 11th default entry
        // this lets it be before "Clear outputs" which is the last entry of the
        // CodeCell contextmenu and plays nicely with the first notebook entry
        // ('Clear all outputs') thus should stay as the last one.
        // see https://github.com/blink1073/jupyterlab/blob/3592afd328116a588e3307b4cdd9bcabc7fe92bb/packages/notebook-extension/src/index.ts#L802
        // TODO: PR bumping rank of clear all outputs instead?
        let notebook_command_manager = new command_manager_1.NotebookCommandManager(app, notebookTracker, 'notebook', 
        // adding a very small number (epsilon) places the group just after 10th entry
        10 + Number.EPSILON, 
        // the group size is increased by one to account for separator,
        // and by another one to prevent exceeding 11th rank by epsilon.
        lsp_commands.length + 2);
        notebook_command_manager.add_context_separator(0);
        notebook_command_manager.add(lsp_commands);
        function updateOptions(settings) {
            // let options = settings.composite;
            // Object.keys(options).forEach((key) => {
            //  if (key === 'modifier') {
            //    // let modifier = options[key] as KeyModifier;
            //    CodeMirrorExtension.modifierKey = modifier;
            //  }
            // });
        }
        settingRegistry
            .load(plugin.id)
            .then(settings => {
            updateOptions(settings);
            settings.changed.connect(() => {
                updateOptions(settings);
            });
        })
            .catch((reason) => {
            console.error(reason.message);
        });
    },
    autoStart: true
};
/**
 * Export the plugin as default.
 */
exports.default = plugin;
