"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const coreutils_1 = require("@jupyterlab/coreutils");
const RE_WIN_PATH = /^file:\/\/\/[a-z]:\//i;
function sleep(timeout) {
    return __awaiter(this, void 0, void 0, function* () {
        return new Promise(resolve => {
            setTimeout(() => {
                resolve();
            }, timeout);
        });
    });
}
exports.sleep = sleep;
// TODO: there is a JL native version of this which uses adaptive rate, maybe use instead?
function until_ready(is_ready, max_retrials = 35, interval = 50, interval_modifier = (i) => i) {
    return new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
        let i = 0;
        while (is_ready() !== true) {
            i += 1;
            if (max_retrials !== -1 && i > max_retrials) {
                reject('Too many retrials');
                break;
            }
            interval = interval_modifier(interval);
            yield sleep(interval);
        }
        resolve(is_ready);
    }));
}
exports.until_ready = until_ready;
/**
 * TODO: this is slightly modified copy paste from jupyterlab-go-to-definition/editors/codemirror/extension.ts;
 */
function getModifierState(event, modifierKey) {
    // Note: Safari does not support getModifierState on MouseEvent, see:
    // https://github.com/krassowski/jupyterlab-go-to-definition/issues/3
    // thus AltGraph and others are not supported on Safari
    // Full list of modifier keys and mappings to physical keys on different OSes:
    // https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/getModifierState
    if (event.getModifierState !== undefined) {
        return event.getModifierState(modifierKey);
    }
    switch (modifierKey) {
        case 'Shift':
            return event.shiftKey;
        case 'Alt':
            return event.altKey;
        case 'Control':
            return event.ctrlKey;
        case 'Meta':
            return event.metaKey;
        default:
            console.warn(`State of the modifier key "${modifierKey}" could not be determined.`);
    }
}
exports.getModifierState = getModifierState;
class DefaultMap extends Map {
    constructor(default_factory, entries) {
        super(entries);
        this.default_factory = default_factory;
    }
    get(k) {
        return this.get_or_create(k);
    }
    get_or_create(k, ...args) {
        if (this.has(k)) {
            return super.get(k);
        }
        else {
            let v = this.default_factory(k, ...args);
            this.set(k, v);
            return v;
        }
    }
}
exports.DefaultMap = DefaultMap;
function server_root_uri() {
    const server_root = coreutils_1.PageConfig.getOption('serverRoot');
    const user_settings = coreutils_1.PageConfig.getOption('userSettingsDir');
    if (server_root.startsWith('~') && user_settings.startsWith('/home/')) {
        return ('file://' +
            server_root.replace('~', user_settings.substring(0, user_settings.indexOf('/', 6))));
    }
    else {
        return 'file://' + server_root;
    }
}
exports.server_root_uri = server_root_uri;
/**
 * compare two URIs, discounting:
 * - drive capitalization
 * TODO: probably use vscode-uri
 */
function uris_equal(a, b) {
    const win_paths = is_win_path(a) && is_win_path(b);
    if (win_paths) {
        return normalize_win_path(a) === normalize_win_path(b);
    }
    else {
        return a === b;
    }
}
exports.uris_equal = uris_equal;
/**
 * grossly detect whether a URI represents a file on a windows drive
 */
function is_win_path(uri) {
    return uri.match(RE_WIN_PATH);
}
exports.is_win_path = is_win_path;
/**
 * lowercase the drive component of a URI
 */
function normalize_win_path(uri) {
    return uri.replace(RE_WIN_PATH, it => it.toLowerCase());
}
exports.normalize_win_path = normalize_win_path;
function uri_to_contents_path(child, parent) {
    parent = parent || server_root_uri();
    if (parent == null) {
        return null;
    }
    if (child.startsWith(parent)) {
        return child.replace(parent, '');
    }
    return null;
}
exports.uri_to_contents_path = uri_to_contents_path;
